/*
 * Created on 10/11/2004
 */
package moca.core.proxy.communication;

import java.io.IOException;
import java.net.InetSocketAddress;

import org.apache.log4j.Logger;

import moca.core.proxy.ProxyException;
import moca.core.proxy.communication.async.ReceiveServerNotification;
import moca.core.proxy.communication.sync.SynchronousUDPClient;
import moca.core.proxy.development.MessageListener;
import moca.core.proxy.message.Message;
import moca.core.proxy.util.ProxyConfiguration;
import moca.protocol.eci.client.ECIClient;
import moca.protocol.net.TCPConnection;
import moca.protocol.net.async.DataListener;
import moca.protocol.net.async.ReceivedData;
import moca.protocol.net.async.tcp.AsyncTCPServer;
import moca.util.eci.ECIConstants;
import moca.util.eci.Topic;


/**
 * Intermediate the communication between the proxy and the server.
 * Provide synchromous and asynchronous commnunication.
 * 
 * @author hana
 *
 */
public class ServerCommunication {

	// comunicao sincrona com o servidor
	//InetSocketAddress syncServerAddr;
	//TODO: Vai poder os dois tipos de protocolo ? TCP e UDP...
	// mas neste caso seria uma porta para cada? e como definir o tipo de protocolo??
	// na hora de enviar?
	
	private TCPConnection myConnection  = null; 
	
	private AsyncTCPServer connection = null ;
	
	private SynchronousTCPServerThread TCPservThread = null ;
	
	// comunicao lado servidor
	ECIClient  esSubscriber;
	//TODO -- clientes tcp e udp	
	
	private InetSocketAddress 	UDPServerSocketAddress 	= null;
	
	SynchronousUDPClient UDPClients = null;
	
   // Por enquanto  s um... precisaria de mais???
	/** Listener for client messages */
	private MessageListener msgListener=null;
	//TODO
	//List listeners; // Lista de listeners que querem receber mensagens
	// ou um nico MessageListener
	private int reconnecting = 0 ;
	private int tcpPort = 23000;
	private boolean freeResource = false ;
	private static final Logger logger = Logger.getLogger(ServerCommunication.class);
	
	String _serverLocalAddr  = ProxyConfiguration.getString("proxy.server.ip_address"); //$NON-NLS-1$
	//String _serverPort = ProxyConfiguration.getString("proxy.sync_server.port"); //$NON-NLS-1$
	String _evServerPort = ProxyConfiguration.getString("proxy.event_server.port"); //$NON-NLS-1$	
	String _tcpServerPort = ProxyConfiguration.getString("proxy.server_tcp.port");	
	String _SeverUdpPort = ProxyConfiguration.getString("proxy.server_udp.port");
	String _proxyTcpPort = ProxyConfiguration.getString("proxy.client_tcp.port");
	String serverAddr = ProxyConfiguration.getString("server.address");
	int tcpServerPort = Integer.parseInt(_tcpServerPort) ;
	int tcpClientPort = Integer.parseInt(_proxyTcpPort) ;
	//	syncServerPort = Integer.parseInt(_serverPort);
	int eventServerPort = Integer.parseInt(_evServerPort);
	 String serverLocalAddress = _serverLocalAddr;
	int serverUDPPort = Integer.parseInt(_SeverUdpPort);
	
	/**
	 * 
	 */
	//proxy.server_udp.port=55200
	
	public ServerCommunication() {
		esSubscriber = null;
	}
	
	public void SetAsynchronousTCPSocket(InetSocketAddress syncServerAddr){
		tcpPort = syncServerAddr.getPort();
		try {
			connection = new AsyncTCPServer(new InetSocketAddress("0.0.0.0",tcpClientPort)) ;
			//connection = new AsyncTCPServer(syncServerAddr) ;
		} catch (IOException e) {
			e.printStackTrace();
		}
		connection.addListener(new AsyncTCPListener()) ;
	}
	
	private class AsyncTCPListener implements DataListener{
		CommunicationProtocol protocol = null ;
		
		public void onReceiveData(ReceivedData arg0) {
			Message msg = (Message)arg0.getData() ;
			try {
				protocol = new CommunicationProtocol(CommunicationProtocol.TCP,CommunicationProtocol.ASYNC);	
			    protocol.setAddress(msg.getClientAddress()) ;
			} catch (ProxyException e) {
				e.printStackTrace();
			}
			
			try {
				protocol.setAsyncTCPConnection(connection) ;
			} catch (IOException e) {
				e.printStackTrace();
			}
			
			notifyReceivedMessage(msg,protocol);
			
		}
		
	}
	public void SetSynchronousTCPSocket(InetSocketAddress syncServerAddr)
	{
		tcpPort = syncServerAddr.getPort();
		do{
			try {
				myConnection  = new TCPConnection( syncServerAddr );
			} catch (IOException e) {
				try {
					Thread.sleep(500) ;
				} catch (InterruptedException e1) {
				}
			}
		}while(myConnection == null || !myConnection.getSocket().isConnected());
			TCPservThread = new SynchronousTCPServerThread();
			TCPservThread.start() ;

	}
	
	public void SetSynchronousUDPSocket(InetSocketAddress syncServerAddr){
		UDPServerSocketAddress = syncServerAddr ;
		UDPClients = new SynchronousUDPClient(UDPServerSocketAddress,this) ;
		UDPClients.start() ;
	}
	
	public void createSubscriberToServer(InetSocketAddress asyncServerAddr, InetSocketAddress  localAddress,String ctype) throws IOException
	{
		if(ctype.equals(CommunicationProtocol.TCP))
		   esSubscriber = new ECIClient(ECIConstants.TCP, asyncServerAddr, localAddress);
		else if(ctype.equals(CommunicationProtocol.UDP))
			 esSubscriber = new ECIClient(ECIConstants.UDP, asyncServerAddr, localAddress);
		else
			throw new IOException("wrong type of protocol. It must be CommunicationProtocol.TCP or CommunicationProtocol.UDP");
	}
	
	
	public void subscribe(String subject) throws ProxyException
	{
		if(esSubscriber == null)
			throw new ProxyException("Asynchronous communication is not possible. Create a subscriber.");

		Topic topic = esSubscriber.subscribe(subject); 
		ReceiveServerNotification listener = new ReceiveServerNotification(this);
		esSubscriber.addListener(listener, topic);
	
	}
	
	public void unsubscribe(String subject)
	{
		if (this.esSubscriber!=null)
			this.esSubscriber.unsubscribe(new Topic(subject));					
	}
	
	public void publishMessage(Message m, String protocol) throws Exception
	{
		if(this.esSubscriber!=null && m.getSubject()!=null ){
			if(protocol.equals(CommunicationProtocol.TCP))
			    this.esSubscriber.publish(m.getSubject(), m,ECIConstants.TCP);
			else if(protocol.equals(CommunicationProtocol.UDP))
				this.esSubscriber.publish(m.getSubject(), m, ECIConstants.UDP);
		}
	}
	
	
	public void sendAsyncMsg(Message m) throws IOException{
		UDPClients.sendMsg(m);
	}
	
	public void sendMsg(Message m) throws IOException{
		if(myConnection != null)
		   myConnection.send(m); 
		else if(connection != null){
			TCPConnection myConnection = null ;
			try {
				myConnection = new TCPConnection(new InetSocketAddress(serverAddr,tcpPort)) ; 
			} catch (IOException e1) {
				logger.warn("Proxy could not send message to server");
			}
			
			try {
				myConnection.send((Message)m) ;
				   
			} catch (IOException e2) {
				System.out.println("> Error when sending a msg to server.");
			}
			finally{
				try {
					if(myConnection != null)
					    myConnection.close() ;
					    myConnection = null ;
				} catch (IOException e) {
					;
				}
			}
		}
	}
	/**
	 * @return Returns the esSubscriber.
	 */
	public ECIClient getEsSubscriber() {
		return esSubscriber;
	}
	
	// Por enquanto s tem um listener de cada tipo.. mas tvz tenha
	// que ter uma lista
	/**
	 *  Register the listener for received messages
	 * @param msgListener The msgListener to set.
	 */
	public void addMessageListener(MessageListener msgListener) {
		this.msgListener = msgListener;
	}
	
	/**
	 * Repass for the aplication the message received from the client
	 * @param msg  The Message
	 * @param protocol Client communication protocol. Note that if the message is
	 *                 from a pub/sub, this protocol only contains the communication mode = ASYNC.
	 */
	public void notifyReceivedMessage(Message msg, CommunicationProtocol protocol ){
		//	TODO: Qdo tiver mais de um: Varrer os listeners para encaminhar a mensagem
		msgListener.onReceiveMessage(msg,protocol);
	}
	
	 private class SynchronousTCPServerThread extends Thread {
			
	    	public void run(){
	    		CommunicationProtocol protocol = null;
				
				while(myConnection != null &&  myConnection.getSocket() != null && myConnection.getSocket().isConnected()){
	    		try {
			
	    			reconnecting = 0 ;
	    			Message msg = (Message)myConnection.receive() ;
					
					protocol = new CommunicationProtocol(CommunicationProtocol.TCP,CommunicationProtocol.SYNC);
					protocol.setSyncTCPConnection(myConnection) ;
					protocol.setAddress(myConnection.getRemoteAddress());
					
					
					notifyReceivedMessage(msg,protocol);
					
				} catch (ClassNotFoundException e) {
					e.printStackTrace();
					break ;
				} catch (IOException e) {
					if(reconnecting > 719){  // waits (719+1) x 0.5 sec = 6 minute for the server to reconnect 
					  logger.error("Error - It was not possible to reconnect to the server application.");
					  System.out.println("Error - It was not possible to reconnect to the server application.");
					  break ;
					}
					  reconnecting++ ;
					    try {
							sleep(500) ;
						} catch (InterruptedException e1) {
							e.printStackTrace();
						}
						try {
							 if(myConnection == null){
								 break ;
							 }
							 myConnection  = new TCPConnection( myConnection.getRemoteAddress() );
						} catch (IOException e1) {
							continue; //
						}
				} catch (ProxyException e) {
					break ;
				}
	    	  }//while
				try {
					if(myConnection != null)
					    myConnection.close() ;
				} catch (IOException e) {
					e.printStackTrace();
				}
	    	}
	    	
	 }// SynchronousServerThread class
		
	 public void freeResources(){
		 
 		try {
 			freeResource = true ;
 			if(UDPClients != null)
 				UDPClients.freeResources() ;
 			
 			if(myConnection != null)
				myConnection.freeResources() ;
 			
			myConnection = null ;
			if(connection != null){
				connection.removeAllListener() ;
				connection.freeResources() ;
			}
			connection = null ;
			
			if(esSubscriber != null){
				esSubscriber.freeResources();
			}
			} catch (IOException e) {
			}
			
 	}
}
