/* ----------------------------------------------------------------------
 * FILE'S INFORMATIONS
 * ----------------------------------------------------------------------
 * Filename.....: UDPSynchronousServer.java
 * Project......: proxy
 * Created on...: 09/10/2004
 * Author.......: hana
 * ----------------------------------------------------------------------
 */
package moca.core.proxy.communication.sync;

import java.io.IOException;
import java.net.DatagramPacket;
import java.net.InetSocketAddress;
import java.net.UnknownHostException;

import moca.core.proxy.ProxyException;
import moca.core.proxy.communication.ClientCommunication;
import moca.core.proxy.communication.CommunicationProtocol;
import moca.core.proxy.message.Message;
import moca.protocol.net.sync.udp.UDPDatagram;

import org.apache.log4j.Category;

/**
 * 
 * @author Hana Rubinsztejn
 */

public class SynchronousUDPServer extends Thread
{
	private UDPDatagram 		server = null;
	private	boolean			isUDPServerRunning = true;
	ClientCommunication clientComm = null;
	
	static   Category			cat = Category.getInstance(SynchronousUDPServer.class.getName());

	/**
	 * @param udpServer
	 */
	public SynchronousUDPServer(UDPDatagram udpServer, ClientCommunication clientComm )
	{
		this.server = udpServer;
		this.clientComm = clientComm;
	}

	/* (non-Javadoc)
	 * @see java.lang.Runnable#run()
	 */
	public void run()
	{
		DatagramPacket		datagramPacket = null;
		NewUDPClient		newUDPClient;
		
		while(isUDPServerRunning)
		{
			try
			{
				datagramPacket = server.receive();
			}
			catch (IOException e)
			{
				cat.warn("Impossible to create one instance of the UDP Server."+e.getStackTrace());
				return;
			}
			
			newUDPClient = new NewUDPClient(datagramPacket);
			newUDPClient.start();
		}
	}

	/**
	 * @return
	 */
	public boolean isUDPServerRunning()
	{
		return isUDPServerRunning;
	}

	/**
	 * @param b
	 */
	public void setUDPServerRunning(boolean b)
	{
		isUDPServerRunning = b;
	}

	
	
	/**
	 *  Handle each message received from UDP protocol
	 * @author hana
	 *
	 */
	private class NewUDPClient extends Thread
	{
		private	DatagramPacket		datagramPacket;
		//private	UDPDatagram			udp;
		
		protected NewUDPClient(DatagramPacket datagram)
		{
			this.datagramPacket = datagram;
		}
				
		/* (non-Javadoc)
		 * @see java.lang.Runnable#run()
		 */
		public void run()
		{
			InetSocketAddress		clientAddress = null;
			Message              msg = null;
			int						clientPort = 0;

			try
			{
				clientPort = datagramPacket.getPort();
				clientAddress = new InetSocketAddress(datagramPacket.getAddress(), clientPort);
				msg = (Message) UDPDatagram.convertBytesArrayToObject(datagramPacket.getData());				
				cat.debug("Handling UDP message: " + msg);
							 
				//Enviar a msg para a aplicao, junto com o protocolo
				CommunicationProtocol cm = new CommunicationProtocol(CommunicationProtocol.UDP,CommunicationProtocol.SYNC);
				cm.setAddress(clientAddress);
				
				//	Encaminhar a mensagem
				clientComm.notifyReceivedMessage(msg,cm);
				
				
			}
			catch (UnknownHostException e)
			{
				cat.error("Host name unknown",e);
			}
			catch (ClassNotFoundException e1)
			{
				cat.error("An error during the conversion of the Message Class",e1);
			}
			catch (IOException e1)
			{
				cat.error("An error during the conversion of the Message Class",e1);
			} catch (ProxyException e) {
				cat.error("Error during CommunicationProtocol initialization" + e);
			}
			finally
			{
				//datagramPacket = null;
			}
		}
	}

	public void freeResources(){
		setUDPServerRunning(false);
		
		if(server != null){
			try {
				server.freeResources() ;
			} catch (IOException e) {}
		}
	}
}
