
package moca.core.proxy.development;

import moca.core.proxy.communication.CommunicationProtocol;
import moca.core.proxy.message.Message;


/**
 * 
 * A MessageListener object is used to get the received message. The
 * message may be sent by a synchronous or asynchronous(publish) communication,
 * what can be verified in the protocol field.
 * 
 * <p>
 * 
 * The proxy's implementation must insure that it passes messages serially to the listener. 
 * This means that a listener can assume that the onReceiveData() method is not 
 * called with the next notification until the last call has been completed.
 * <p>
 * It is very important to understand that each message received is repassed 
 * to all objects which has been added 
 * (through the {@link moca.core.proxy.development.ProxyFacade#addListener(MessageListener)} method) to deal 
 * with this message.
 * They are called serially. Be careful, do not implement something that probably would take a long time 
 * to be completed. If this is necessary, it is highly recommended to create a thread for that. Otherwise, the following
 * listeners could not be called until the current task has been finished.   
 * 
 * @author Hana Rubinsztejn
 * 
 */
public interface MessageListener {
	
	/**
    * The listener method. Its called when the message is received.
    * 
    * @param data - The received Message.
    * @param protocol - The communication protocol used by the client.
    *                   If it is a published message (pub/sub) from a client 
    *                   the protocol only has the information that it is a pub/sub communication
    * 						@link moca.core.proxy.communication.CommunicationProtocol#communicMode is 
    *                     @link moca.core.proxy.communication.CommunicationProtocol#ASYNC.
    */
   abstract public void onReceiveMessage(Message data, CommunicationProtocol protocol);
   // Qdo for uma publicao, no se sabe o cliente que publicou, apenas
	// para qual assunto deva ser publicado
	// Assim o Communication Protocol, ter apenas o modo da comunicao,
	// para indicar isso.
}
