/*
 * Created on 29/11/2004
 *
 * TODO To change the template for this generated file go to
 * Window - Preferences - Java - Code Style - Code Templates
 */
package moca.core.proxy.development;


import java.io.IOException;

import moca.core.proxy.ProxyException;
import moca.core.proxy.communication.CommunicationProtocol;
import moca.core.proxy.message.Message;

/**
 * Main interface between proxy application and the framework.
 * <p>
 * The proxy application must use the methods of this class to
 * intialize the proxy framework and to use the faciliies it offers.
 * 
 * <p>
 *  The proxy intermediates all the communication between server and clients of an applicatiom.
 *  The proxy framework provides ways to configure cache policies and rules of adaptations
 *  acording to the context of the clients. These configuration rules must be specified
 *  in a XML file. This file links each context to one or more actions (adaptaions) to be 
 *  executed when the client (the message receiver) is in that state (the context).   
 * 
 *  Messages sent to clients may be adapted or cached, acording to the configured policies. *  
 * 
 *  <p>
 *  First it is necessary to register the proxy in the MoCA, using 
 *  {@link  #registerProxyInMoCA} method, and passing the parameters of the application, 
 *  like its name and keywords.
 * 
 * <p>
 *  <b> The proxy application is responsible for maintaining the list of clients 
 *  that the proxy intermediates the communication. </b>
 *  Use {@link  #addClient(String, CommunicationProtocol)} and {@link #removeClient(String)} 
 *  methods to do so.
 * 
 *  <p>
 *   For the clients that the application wants to monitor the context, it is necessary
 *  to make their registration on MoCA CIS. For this purpose use the methods 
 *  {@link #registerClientAtCIS(String)} and {@link #removeClientFromCIS(String)}. 
 * 
 *   Note that remove clients from MoCA CIS do not remove them from the proxy framework. 
 *   But remove clients from framework, automaticaly removes them from MoCA.
 *  
 *  <p> 
 *   The messages to clients may be adapted, if necessary. But messages to the server
 *   do not suffer any kind of addaption. Due to this, the way to send messages to the server or clients
 *   are different. To send messages to clients use {@link #sendMsgToClient}, and to server
 *  use {@link #sendMsgToServer(Message)}. 
 * 
 *  <p>
 *   The proxy framework receives the messages and pass them to the proxy application 
 *  through listeners, calling the {@link MessageListener#onReceiveData(Message, CommunicationProtocol)}.
 *   So the proxy application must define <code>MessageListeners </code>
 *   to handle these messages, and register it in the framework using 
 *  {@link #addReceiveListener(MessageListener)} method.
 * 
 *   
 *  <p> 
 *  As an example, when a client send a message for the first time, the proxy aplication
 *  will receive it throught the <code>MessageListener.onReceiveData</code> method.
 *  The application may verify if it is an authorized user, for example, and after that
 *  it may add this new client to the list of served clients (using  the <code>addClient </code> method).
 *  The message may be re-transmitted to the server, and to do this, the method <code>
 *  sendMsgToServer </code> should be invoked. 
 *  
 * @author Hana Rubinsztejn
 * 
 */
public interface ProxyFacade {
	
		
	// Interface com MoCA
	
   // cadastro no DS??
	/**
	 * Register the service at MoCA DS. Two services will be registered,
	 *  serviceName_SYNC e serviceName_ASYNC
	 * @param serviceName
	 */
	public void registerProxyInMoCA(String serviceName);
	
// TODO: tvz colocar a inicializao do proxy . indicando quais os protocolos
	// de comunicao devam ser instanciados.. (de ambos os lados)

	/**
	 *  Configure and initiates the proxy. Configure communication with 
	 *  application server and clients, and with MoCA CIS. Moreover,
	 *  the configuration XML file is loaded and the context and corresponding
	 *  actions are set.  After all this the proxy is prepared to execute, and 
	 *  is started.
	 *  <p>
	 *  The application server address must
	 *  be provided. Note that the proxy may communicate with the server via a
	 *  synchronous(TCP/UDP) and/or assynchronous way. If it is not necessary
	 *  to have both kinds of communication, set the undesired one as null.
	 *  But note that at least one must be not null.
	 * 
	 * @param syncServerAddress  Address for synchronous communication
	 * @param asyncServerAddress Address for asynchronous communication
	 * @param ctype - defines the connection type TCP or UDP
	 */
	public void init(String proto, String mode);
	
	/**
	 * The client will be registered at CIS, so that its contexts can be monitored.
	 *  Note that presently the identification of a client at CIS is its MAC Address.
	 * @param client The MAC Address identifying the client.
	 */
	public void registerClientAtCIS(String client);
	
	/** 
	 * Remove client registration from CIS. Note that the client is not removed from the
	 * list of clients handled by the proxy.  
	 * @param client Client's MAC Address
	 */
	public void removeClientFromCIS(String client);
		
	/**
	 *  Remove all clients from CIS. Note that the clients are not removed from the proxy`s clients list.
	 */
	public void removeAllClientsFromCIS();
	
	
	// Manuteno da lista de clientes
	
	/**
	 * Add a client on the list of clients handled by the proxy system. 
	 * After this, the proxy will intermediate the communication
	 *  between the server and this client.
	 * @param newClient Client identification, commonly its MACAddress
	 * @param protocol Communication protocol for the proxy uses to communicate with the client
	 */
	public void addClient(String newClient, CommunicationProtocol protocol);

	/**
	 * Add a client on the list of clients handled by the proxy system. After this, the proxy will intermediate the communication
	 *  between the server and this client, making adaptations if necessary(required).
	 *  If requested the client will also be registered at CIS.
	 * @param newClient Client identification,  its MACAddress
	 * @param protocol Communication protocol for the proxy uses to communicate with the client
	 * @param registerAtCIS True if the client must be registered at CIS, false, otherwise.
	 */
	public void addClient(String newClient, CommunicationProtocol protocol, boolean registerAtCIS);
	
	/**
	 * Remove the client from the list of the clients managed by the proxy. 
	 * The client will also be removed from CIS.
	 * @param clientId Client`s MAC Address
	 */
	public void removeClient(String clientId);
		
	/**
	 *  Remove all clients from the system, and from CIS.
	 */
	public void removeAllClients();
	
	
	// Comunicao - envio e recebimento de mensagens

	// Mensagens de/para clientes
	/**
	 * Send the message to the client specified in the Message's addressee attribute.
	 *  According to the current context of the client the message may be adapted. 
	 * @param  message  Message that will be sent. Note that the addressee must be specified.
	 * @throws ProxyException If the message addressee is not specified.
	 */
	public void sendMsgToClient(Message message) throws ProxyException;

	/**
	 * Publish the message for all clients interested in the subject specified in the message.
	 * According to the current context of each client, the message may be adapted.
	 * @param message Message to be published. Note that the subject attribute must be set.
	 * @throws ProxyException If the subject is not specified.
	 */
	public void publishMsgToClients(Message message) throws ProxyException;
	
	
	/**
	 * Send the message to the server.
	 * @param  message  Message that will be sent. Note that the addressee must be specified.
	 
	 */
	public void sendMsgToServer(Message m, CommunicationProtocol protocol); 
	
	/**
	 * 
	 * @param message  Message that will be sent.
	 * @param  clientProtocol Communication protocol that the proxy uses to communicate with the client.
	 */
	public void publishMsgToServer(Message message, CommunicationProtocol protocol);
	
	public void subscribeToServer(String subject);
	public void unsubscribeFromServer(String subject);
	
	/**
	 *  Register a listener to receive messages from server.
	 * <p>
	 *  To receive messages from server, the application has to implement
	 *  a listener, and register it in the proxy. 
	 * 
	 * @param listener A {@link moca.core.proxy.development.MessageListener}
	 */
	//TODO: indicar tipo de protocolo??????
	public void addServerMsgListener(MessageListener listener);
	
	/**
	 *  Register a listener to receive messages from clients.
	 * <p>
	 *  To receive messages from clients, the application has to implement
	 *  listener, and register it in the proxy. 
	 * 
	 * @param listener A {@link moca.core.proxy.development.MessageListener}
	 */
	//TODO: indicar o tipo de protocolo??????
	public void addClientMsgListener(MessageListener listener);
	
	/**
	 *  Register the listener for the application proxy be able to receive 
	 *  notifications of the connectivity status of clients.
	 * <p>
	 *  The application must register the listener to receive the connection
	 *  notification, that informs the client and its current situation (connected, or not).
	 * <p>
	 *  The proxy application may use this information to decide, for example,
	 *  when a client may not be served anymore (after a timeout period of disconnectio),
	 *  and then remove it from the proxy (using @link #removeClient(String)).
	 * 
	 * @param listener A {@link moca.core.proxy.development.ConnectionListener}
	 */
	 public void addClientConnectedListener(ConnectionListener listener);
	 
	 
	 /**
	  * Free all resources allocated by the proxy during its execution
	  * @throws IOException
	  */
	 public void freeResources();
	 
}
