
package moca.core.proxy.message;

import java.net.InetSocketAddress;

import moca.core.proxy.receivers.ReceiverMessage;


/**
 *  Definition of the message accepted by the proxy. 
 *  The message must contain some information (like the adressee) that the proxy needs to 
 *  intermediate the communication between aplication's server and clients, 
 *  and perform the adaptations when nescessary.   
 * 
 * @author hana
 */
public interface Message extends ReceiverMessage, Cloneable {
	
	/** Messages types */
	public static final int DATA = 0;
	public static final int CONTROL = 1;

	/** Special data type that indicates that the data sent in the message is an  
	 * application specific object, so its is not adaptable.
	 */
	public static final String OBJECT = "object";
	
	
	/**
	 * @return Returns the addressee.
	 */
	public abstract String getAddressee();

	/**
	 * @param addressee The addressee to set.
	 */
	public abstract void setAddressee(String addressee);

	/**
	 * @param socket the address where the tcp asynchronous client wants to receive connections  
	 */
	public abstract void setClientAddress(InetSocketAddress socket);
	
	/**
	 * @return the address where the tcp asynchronous client wants to receive connections
	 */
	public abstract InetSocketAddress getClientAddress();
	
	/**
	 * @return Returns the data.
	 */
	public abstract byte[] getData();

	/**
	 * @param data The data to set.
	 */
	public abstract void setData(byte[] data);

	/**
	 * @return Returns the sender.
	 */
	public abstract String getSender();

	/**
	 * @param sender The sender to set.
	 */
	public abstract void setSender(String sender);

	/**
	 * @param dataType The dataType to set.
	 */
	public abstract void setDataType(String dataType);

	/**
	 * @param msgType The msgType to set.
	 */
	public abstract void setMsgType(int msgType);

	/**
	 * @return Returns the dataType. This type is 
	 *  a string similar to the MIME format, or a special tag OBJECT ="object"
	 */
	public abstract String getDataType();

	/**
	 * @return Returns the msgType - Message.DATA or Message.CONTROL.
	 */
	public abstract int getMsgType();

	/**
	 * @return Returns the subject.
	 */
	public abstract String getSubject();

	/**
	 * @param subject The subject to set.
	 */
	public abstract void setSubject(String topic);
	/**
	 *  Clone the object
	 */
	public Object clone() throws CloneNotSupportedException ;
}