package moca.core.proxy.receivers;

import moca.core.proxy.ClientInfo;
import moca.core.proxy.actions.adapters.AdaptationException;
import moca.core.proxy.actions.adapters.Adapter;
import moca.core.proxy.message.Message;
import moca.core.proxy.state.State;

import org.apache.log4j.Logger;

/**
 * Gerencia as adaptaes de mensagem. Cada mensagem a ser processada  submetida
 * uma srie de adapters, que analisam (e provavelmente alteram) o seu contedo. 
 * @author hana
 */
public class AdapterManager extends Receiver {
	/**
	 * Logger para o receiver
	 */
	private static final Logger LOGGER = Logger.getLogger(AdapterManager.class);
	/**
	 * Processa uma mensagem
	 * @param clientInfo informaes do destinatrio
	 * @param msg mensagem
	 * @return retorna um par (mensage adaptada, proximo receiver)
	 */
	protected ReceiverResult process(ClientInfo clientInfo, ReceiverMessage msg) {
		Adapter adapter = null;
		try {
			//itera cada um dos estados a serem avaliados
			for (int i = 0; i < clientInfo.getNumStates(); i++) {
				State st = clientInfo.getState(i);
		LOGGER.debug("Cliente: " + clientInfo.getClientId() + " " + "Estado " + 
				      st.getStateConf().getExpression().getValue() +
				      " is " + st.isOn());   //%%%%
				//recupera o adapter para o estado
				adapter = st.getAdapter();
				//executa o adapter 
				msg = adapter.execute(clientInfo, (Message) msg);
				if (msg == null) {
		LOGGER.debug( "Mensagem retornada nula...fluxo de adaptaes interrompido no estado " + st.getStateConf().getExpression().getValue()); //%%%%%%   
					//finalize a adaptao se o ultimo adapter retornou null
					break;
				}
			}
		} catch (AdaptationException e) {
			//TODO Ser que cancelo ou continuo????  
			
			//caso tenha ocorrido alguma excees, finalize a adaptao para esta
			//mensagem
			LOGGER.warn(
				"Erro na execuao de " + adapter.getClass() + ".execute()",
				e);
		}
		return new ReceiverResult(msg, Sender.class);
	}
}
