package moca.core.proxy.receivers;

/**
 * Mensagem de controle para o receiver CacheManager.
 * @author hana
 *
 */
public class CacheMessage implements ReceiverMessage {
	/**
	* Interrompe o cache
	*/
	public static final int CACHE_STOP=1;
	/**
	 * Inicia o cache
	 */
	public static final int CACHE_START=2;
	/**
	 * Mensagem de cliente para avaliao (se ser ou no 'cacheada') 
	 */
	public static final int CACHE_MESSAGE=3;
	/**
	 * Destroi as mensagens do cache
	 */
	public static final int CACHE_DESTROY=4;
	/**
	 * Armazena o pacote contendo a mensagem do cliente, caso o tipo seja CACHE_MESSAGE
	 */
	private Package pack;
	/**
	 * Tipo da mensagem. Pode ser:
	 * CACHE_STAR - inicia o cache
	 * CACHE_STOP - finaliza o cache
	 * CACHE_MESSAGE - mensagem de cliente a ser avaliada
	 * CACHE_DESTROY - destroi as mensagens do cache
	 */
	private int type;
	/**
	 * Nome da classe que ir gerenciar o cache
	 */
	private String cacheClassName;
	/**
	 * Construtor para as mensagens do tipo CACHE_MESSAGE
	 * @param pack pacote contendo mensagem de cliente
	 */
	public CacheMessage(Package pack) {
		this.pack = pack;
		this.type = CACHE_MESSAGE;
	}
	/**
	 * Construtor para mensagem do tipo CACHE_STOP ou CACHE_DESTROY
	 */
	public CacheMessage(int type) {
		this.type = type;
	}
	/**
	 * Construtor para mensagens do tipo CACHE_START
	 * @param cacheClassName nome da classe que ir armazenar as mensagens
	 * em cache. Implementa uma poltica de cache.
	 */
	public CacheMessage(String cacheClassName) {
		this.cacheClassName = cacheClassName;
		this.type = CACHE_START;
	}
	/**
	 * Recupera o nome da classe que armazenar as mensagens em cache
	 * @return
	 */
	public String getCacheClassName() {
		return this.cacheClassName;
	}
	/**
	 * Tipo da mensagem
	 * @return tipo da mensagem (CACHE_START, CACHE_STOP, CACHE_MESSAGE)
	 */	
	public int getType() {
		return this.type;
	}
	public Package getPackage() {
		return this.pack;
	}

}
