package moca.core.proxy.state;

import java.io.InputStream;
import moca.core.proxy.state.xml.ConfiguratorXML;

/**
 * Classe base para classes que lem e armazenam informaes 
 * cadastradas no arquivo de configurao do proxy.
 * @author hana
 */
public abstract class Configurator {
	/**
	* Array com os estados a serem monitorados
	*/
	private StateConf[] stateConfs;
	/**
	 * L os estados cadastrados no arquivo de configuracoes
	 * @return
	 */
	protected abstract StateConf[] read();
	/**
	 * Recupera o array de estados a serem monitorados
	 * @return
	 */
	public StateConf[] getStateConfArray() {
		if (stateConfs == null) {
			stateConfs = this.read();
		}
		return this.stateConfs;
	}
	/**
	 * Faz o parser do arquivo de configuracoes
	 * @param is
	 */
	public abstract void parse(InputStream is);

	/**
	 * Instancia unica do configurador (singleton)
	 */
	private static Configurator instance;
	/**
	 * Recupera a instancia unica do Configurador
	 * @return Configurator
	 */
	public static Configurator getInstance() {
		if (instance == null) {
			instance = new ConfiguratorXML();
			InputStream is =
				Configurator.class.getResourceAsStream("/ProxyConfig.xml");
			instance.parse(is);
		}
		return instance;
	}

}
