package moca.core.proxy.state.xml;

import java.io.InputStream;
import java.util.ArrayList;
import java.util.List;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import moca.core.proxy.state.Configurator;
import moca.core.proxy.state.StateConf;
import moca.core.proxy.util.Log4jInitializer;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.w3c.dom.Text;

/**
 * Le um XML com as configuraes do proxy e armazena em estruturas de dados
 * apropriadas.
 * @author hana
 *
 */
public class ConfiguratorXML extends Configurator {
	
	/** XSchema property name */
	private static final String JAXP_SCHEMA_LANGUAGE =
			"http://java.sun.com/xml/jaxp/properties/schemaLanguage";

	/** XSchema property value */
	private static final String W3C_XML_SCHEMA =
			"http://www.w3.org/2001/XMLSchema";
			
	
	static final String JAXP_SCHEMA_SOURCE =
	  "http://java.sun.com/xml/jaxp/properties/schemaSource";


	/**
	 * Le o valor de um elemento
	 * @param el
	 * @return
	 */
	static String getElementValue(Element el) {
		NodeList list = el.getChildNodes();
		StringBuffer buf = new StringBuffer();
		for (int i = 0; i < list.getLength(); i++) {
			Node node = list.item(i) ;
			if (node instanceof Text) {
				Text txtNode = (Text) node;
				buf.append(txtNode.getData());
			} 
		}
		
		if (buf.length() == 0) {
			return null;
		}

		return buf.toString();
	}
	
	private Document doc;

	public void parse(InputStream is) {
		DocumentBuilderFactory bf = DocumentBuilderFactory.newInstance();
		try {
			//bf.setValidating(true);
			
			//bf.setAttribute(JAXP_SCHEMA_LANGUAGE,W3C_XML_SCHEMA);
			//InputStream d = this.getClass().getResourceAsStream("/ProxyConfig.xsd");
			
			//bf.setAttribute(JAXP_SCHEMA_SOURCE, d);

			DocumentBuilder db = bf.newDocumentBuilder();			
			doc = db.parse(is);
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			throw new RuntimeException(e);
		}
	}
	protected StateConf[] read() {
		//deve ler uma sequencia de states
		List stateConfAuxList = new ArrayList();
		NodeList nodeList = doc.getDocumentElement().getChildNodes();
		for (int i = 0; i < nodeList.getLength(); i++) {
			Node node = nodeList.item(i);
			if (node instanceof Element) {
				Element el = (Element) node;
				StateConf stateConf = StateConfXMLReader.read(el);
				if (stateConf != null) {
					stateConfAuxList.add(stateConf);
				}
			}
		}
		return (StateConf[]) stateConfAuxList.toArray(new StateConf[0]);
	}
	
	public static void main(String[] args) throws Exception {
	    Log4jInitializer.init(".");
		ConfiguratorXML conf = new ConfiguratorXML();
		InputStream is = Configurator.class.getResourceAsStream("/ProxyConfig.xml");
		conf.parse(is);
		StateConf[] stateConfs = conf.getStateConfArray();
		
	}

}
