package moca.core.proxy.state.xml;

import moca.core.proxy.state.ActionConf;
import moca.core.proxy.state.Expression;
import moca.core.proxy.state.Rule;
import moca.core.proxy.state.StateConf;
import org.apache.log4j.Logger;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

/**
 * @author hana
 */
public class StateConfXMLReader {
	private static final Logger LOGGER = Logger.getLogger(ActionConfXMLReader.class);

	public static StateConf read(Element el) {
		if (!el.getNodeName().equals("State")) {
			LOGGER.error("Tag State esperada. Encontrada: " + el.getNodeName());
			return null;
		}
		//itere nos filhos da tag State
		NodeList nodeList = el.getChildNodes();
		int i;
		StateConf stateConf = null;
		//localize o expression
		for (i = 0; i < nodeList.getLength(); i++) {
			Node node = nodeList.item(i);
			if (node instanceof Element) {
				//Um state deve ter uma e apenas uma expression
				Expression exp = ExpressionXMLReader.read((Element) node);
				if (exp != null) {
					stateConf = new StateConf(exp);
				}
				break;
			}
		}
		if (stateConf == null) {
			LOGGER.error("State nao possui expression!");
			return null;
		}
		//continue iterando nos filhos da tag state em busca de possiveis actions
		//Obs: Um state pode der um nmero arbitrario de actions (inclusive nenhuma)		
		for (i++; i < nodeList.getLength(); i++) {
			Node node = nodeList.item(i);
			if (node instanceof Element) {
				ActionConf action = ActionConfXMLReader.read((Element) node);
				if (action == null) {
					i--;
					break;					
				}
				action.setType(ActionConf.LISTENER);
				stateConf.addAction(action);
			}
		}	
		//continue iterando nos filhos da tag state em busca das regras. 
		//Obs: Um state pode ter de 1 a n tags Rule
		for (i++; i < nodeList.getLength(); i++) {
			Node node = nodeList.item(i);
			if (node instanceof Element) {
				Rule rule = RuleXMLReader.read((Element) node);
				if (rule != null) {
					stateConf.addRule(rule);
				}
			}
		}
		return stateConf;
	}

}
